@@ -1,4 +1,4 @@
-// $Id: main_thread.c,v 1.6 2006/02/07 11:35:41 khlut Exp $
+// $Id: main_thread.c,v 1.8 2006/02/13 17:42:46 khlut Exp $
 // Copyright 2005 Elphel, Inc.
 //
 // This file is part of GenReS.
@@ -31,6 +31,7 @@
 
 NPNetscapeFuncs NPN;
 NPIdentifier ready,	run;
+int debug=1;
 static int instance_cnt;
 
 /*
@@ -72,19 +73,19 @@ char * objectprop(NPP instance, int object_type, const char *name)
     NPObject *a=0; char  *s0, *s1, *n;
     NPIdentifier id;
     NPVariant var={NPVariantType_Void};
-    printf(" objectprop\n");
+    Debug(" objectprop\n");
     if(NPN.getvalue(instance, object_type, &a))	return 0;
-    printf(" %d\n",a->referenceCount);
+    Debug(" %d\n",a->referenceCount);
     s0=n=strdup(name);
     do {
 	s1=strchr(s0,'.');
 	if(s1) *s1++=0;
 	id=NPN.getstringidentifier(s0);
-	printf("  %s (%x)\n",s0,(int)id);
+	Debug("  %s (%x)\n",s0,(int)id);
 	if(!NPN.hasproperty(instance,a,id)) 	break;
 	if(!NPN.getproperty(instance,a,id,&var))	break;
 	NPN.releaseobject(a); a=0;
-	printf("  %d\n",var.type);
+	Debug("  %d\n",var.type);
 	if(var.type!=NPVariantType_Object)	break;
 	a=var.value.objectValue;
 	var.type=NPVariantType_Void;
@@ -106,7 +107,7 @@ void send_value(PD *pd, char *key, char *value) //send variable to slave
   PR_Lock(pd->lock);
   if(( (!pd->slavectrl) || fprintf(pd->slavectrl,"%s=%s\n",key,value)<=0 ) && pd->status==Running // try to reanimate slave
      && ( strcmp(key,"run") || strcmp(value,"0") )) { 	// ignore run=0 when slave is nonfunctional
-    perror("write to pipe");
+    if(debug)perror("write to pipe");
    
     PR_Unlock(pd->lock);
     wait_slave(pd);
@@ -123,7 +124,7 @@ void send_value(PD *pd, char *key, char *value) //send variable to slave
 	if(pd->vars->var[i].key==run || pd->vars->var[i].key==ready) continue;
 	char *sv=NPVariant_to_string(&(pd->vars->var[i].value));
 	fprintf(pd->slavectrl,"%s=%s\n",NPN.utf8fromidentifier(pd->vars->var[i].key), sv);
-	printf("send %s=%s\n",NPN.utf8fromidentifier(pd->vars->var[i].key), sv);
+	Debug("send %s=%s\n",NPN.utf8fromidentifier(pd->vars->var[i].key), sv);
 	free(sv);
     }
     pd->xid && fprintf(pd->slavectrl,"xid=%x\n",(unsigned)pd->xid);
@@ -187,7 +188,7 @@ typedef struct {
 NPObject * NPAllocate(NPP npp, NPClass *aClass)
 {
     PO *po=PR_Malloc(sizeof(PO));
-    printf("OBJ-ALLOCATE\n");
+    Debug("OBJ-ALLOCATE\n");
     if(po) {
 	po->npo._class=&genresClass;
 	po->npo.referenceCount=0;
@@ -197,31 +198,31 @@ NPObject * NPAllocate(NPP npp, NPClass *aClass)
 }
 void NPDeallocate(NPObject *npobj)
 {
-    printf("OBJ-DEALLOCATE %d\n", npobj? npobj->referenceCount : -100);
+    Debug("OBJ-DEALLOCATE %d\n", npobj? npobj->referenceCount : -100);
     PR_Free(npobj);
 }
 bool NPHasMethod(NPObject *npobj, NPIdentifier name)
 {
-    printf("Has method ");
+    Debug("Has method ");
     if(NPN.identifierisstring(name)) {
         char *s=NPN.utf8fromidentifier(name);
-	printf("string %s\n",s);
+	Debug("string %s\n",s);
 	PR_Free(s);
     }else{
-	printf("int %d\n",NPN.intfromidentifier(name));
+	Debug("int %d\n",NPN.intfromidentifier(name));
     }
     return false;
 }
 bool NPHasProperty(NPObject *npobj, NPIdentifier name)
 {
     NPP instance=((PO*)npobj)->instance;
-    printf("Has property ");
+    Debug("Has property ");
     if(NPN.identifierisstring(name)) {
         char *s=NPN.utf8fromidentifier(name);
-	printf("string %s ",s);
+	Debug("string %s ",s);
 	PR_Free(s);
     }else{
-	printf("int %d ",NPN.intfromidentifier(name));
+	Debug("int %d ",NPN.intfromidentifier(name));
 	return false;
     }
     
@@ -231,10 +232,10 @@ bool NPHasProperty(NPObject *npobj, NPIdentifier name)
 bool NPGetProperty(NPObject *npobj, NPIdentifier name, NPVariant *result)
 {
     NPP instance=((PO*)npobj)->instance;
-    printf("Get property");
+    Debug("Get property");
     NPVariant *var=storage_get(Pd->vars, name);
-    if(!var) {printf(" no\n"); return false;}
-    printf(" yes\n");
+    if(!var) {Debug(" no\n"); return false;}
+    Debug(" yes\n");
     return NPVariant_cpy(result, var);
 }
 
@@ -242,7 +243,7 @@ bool NPSetProperty(NPObject *npobj, NPIdentifier name, const NPVariant *value)
 {
     NPP instance=((PO*)npobj)->instance;
     char *sv=NPVariant_to_string(value);
-    printf("Set property %s %s\n",NPN.utf8fromidentifier(name), sv);
+    Debug("Set property %s %s\n",NPN.utf8fromidentifier(name), sv);
     send_value(Pd, NPN.utf8fromidentifier(name), sv);
     free(sv);
     return true;
@@ -254,21 +255,21 @@ bool NPSetProperty(NPObject *npobj, NPIdentifier name, const NPVariant *value)
 
 char *NP_GetMIMEDescription(void)
 {
-  printf("NP_GetMIMEDescription\n");
+  Debug("NP_GetMIMEDescription\n");
   return get_mime_list();
 }
 
 NPError NP_GetValue(void *future, NPPVariable variable, void *value)
 {
   NPError err = NPERR_NO_ERROR;
-  printf("NP_GETVALUE\n");
+  Debug("NP_GETVALUE\n");
   switch (variable) {
     case NPPVpluginNameString:
-      *((char **) value) = "The GenReS Mozilla plugin V 0.8.1";
+      *((char **) value) = "The GenReS Mozilla plugin V 0.8.3";
       break;
     case NPPVpluginDescriptionString:
       *((char **) value) = "<a href=https://savannah.nongnu.org/projects/genres>GENeric REconfigureable Scriptable plugin</a> for Mozilla/Firefox<br>"
-    			   "Version 0.8.1<br>"
+    			   "Version 0.8.3<br>"
 			   "Author: Sergey Khlutchin<br>"
 			   LICENSE;
       break;
@@ -280,7 +281,9 @@ NPError NP_GetValue(void *future, NPPVariable variable, void *value)
 
 NPError NP_Initialize(NPNetscapeFuncs * moz_funcs, NPPluginFuncs * plugin_funcs)
 {
-  printf("NP_Initialize\n");
+  void *ServiceManager;
+  
+  Debug("NP_Initialize\n");
   if (moz_funcs == NULL || plugin_funcs == NULL)
     return NPERR_INVALID_FUNCTABLE_ERROR;
 
@@ -292,9 +295,12 @@ NPError NP_Initialize(NPNetscapeFuncs * moz_funcs, NPPluginFuncs * plugin_funcs)
     return NPERR_INVALID_FUNCTABLE_ERROR;
 
   NPN= *moz_funcs;
-  ready=NPN.getstringidentifier("ready");
-  ready=NPN.getstringidentifier("run");
+  ready=   NPN.getstringidentifier("ready");
+  run=     NPN.getstringidentifier("run");
   instance_cnt=0;
+  
+  NPN.getvalue(0, NPNVserviceManager, &ServiceManager);
+  init_qservice(ServiceManager);
 
   plugin_funcs->size = sizeof (NPPluginFuncs);
   plugin_funcs->version = (NP_VERSION_MAJOR << 8) + NP_VERSION_MINOR;
@@ -318,11 +324,12 @@ NPError NP_Initialize(NPNetscapeFuncs * moz_funcs, NPPluginFuncs * plugin_funcs)
 
 NPError NP_Shutdown(void)
 {
-  printf("NP_ShutDown\n");
+  Debug("NP_ShutDown\n");
   if(instance_cnt) {
-    printf("Have non-terminated instance, can not shut down!\n");
+    Debug("Have non-terminated instance, can not shut down!\n");
     return NPERR_GENERIC_ERROR;
   }
+  shut_qservice();
   return NPERR_NO_ERROR;
 }
 
@@ -335,7 +342,7 @@ NPError NPP_New(NPMIMEType mime, NPP instance, uint16 mode, int16 argc,
 {
     int i; PD *pd;
     
-    printf("NPP_NEW %s %x %x %x %x\n", mime, (int)instance, (int)mode, (int)argc, (int)saved );
+    Debug("NPP_NEW %s %x %x %x %x\n", mime, (int)instance, (int)mode, (int)argc, (int)saved );
     
     if(!( pd=PR_MALLOC(sizeof *pd) ))	return NPERR_OUT_OF_MEMORY_ERROR;
     if(!( pd->vars=storage_new() ))	{ PR_Free(pd); return NPERR_OUT_OF_MEMORY_ERROR; }
@@ -364,18 +371,18 @@ NPError NPP_New(NPMIMEType mime, NPP instance, uint16 mode, int16 argc,
     for(i=0; i<argc; i++) send_value(pd,argn[i],argv[i]);
     pd->status=Running;
     
-    printf("pdata refcnt=%d\n", Pd->refcnt);
+    Debug("pdata refcnt=%d\n", Pd->refcnt);
     process_pending_events(Pd);
-    printf("pdata refcnt=%d\n", Pd->refcnt);
+    Debug("pdata refcnt=%d\n", Pd->refcnt);
     process_pending_events(Pd);
-    printf("pdata refcnt=%d\n", Pd->refcnt);
+    Debug("pdata refcnt=%d\n", Pd->refcnt);
     
     return NPERR_NO_ERROR;
 }
 
 void free_pd(PD *pd) { // free plugin data if reference counter zeroes
     if(--pd->refcnt)	{PR_Unlock(pd->lock); return;}
-    printf("free pd\n");
+    Debug("free pd\n");
     storage_free(pd->vars);
     free(pd->mime);
     PR_Unlock(pd->lock);
@@ -388,42 +395,42 @@ void wait_slave(PD * pd) { // wait for slave termination
     NPVariant *var;	int i;
     do{
         PR_Sleep(100);
-        printf("Process events %x\n", (int)pd->instance);
+        Debug("Process events %x\n", (int)pd->instance);
         process_pending_events(pd);
         var=storage_get(pd->vars, ready);
         if(!var || var->type!=NPVariantType_String) {
-            printf("variable \"ready\" not found\n");
+            Debug("variable \"ready\" not found\n");
             break;
         }
     }while(var->value.stringValue.utf8characters[0]=='1');
 
-    printf("pdata refcnt=%d\n", pd->refcnt);
+    Debug("pdata refcnt=%d\n", pd->refcnt);
     
     pd->status=Cancelling;
     PR_Sleep(100);
     process_pending_events(pd);
-    printf("wait %d\n",pd->slave);
+    Debug("wait %d\n",pd->slave);
     if(waitpid(pd->slave,&i,WNOHANG)<0) perror("wait");
-    printf("wait observer\n");
+    Debug("wait observer\n");
     if(pd->observer) {PR_JoinThread(pd->observer); pd->observer=0;}
     if(pd->slavectrl) {fclose(pd->slavectrl); pd->slavectrl=0;}
-    printf("Process events %x\n", (int)pd->instance);
+    Debug("Process events %x\n", (int)pd->instance);
     process_pending_events(pd);
-    printf("pdata refcnt=%d\n", pd->refcnt);
+    Debug("pdata refcnt=%d\n", pd->refcnt);
     process_pending_events(pd);
-    printf("pdata refcnt=%d\n", pd->refcnt);
+    Debug("pdata refcnt=%d\n", pd->refcnt);
 }
 
 NPError NPP_Destroy (NPP instance, NPSavedData** save)
 {
-    printf("NP_Destroy\n");
+    Debug("NP_Destroy\n");
     if(Pd) {
       if(Pd->observer && Pd->status==Running) {
 	send_value_hex(Pd,"run", 0);
 	wait_slave(Pd);
       }
       PR_Lock(Pd->lock);
-        if(Pd->refcnt>1) { printf("Have references!\n"); PR_Unlock(Pd->lock); return NPERR_GENERIC_ERROR; }
+        if(Pd->refcnt>1) { Debug("Have references!\n"); PR_Unlock(Pd->lock); return NPERR_GENERIC_ERROR; }
         Pd->instance=0;
       free_pd(Pd);
       instance->pdata=0;
@@ -433,7 +440,7 @@ NPError NPP_Destroy (NPP instance, NPSavedData** save)
 
 NPError NPP_GetValue(NPP instance, NPPVariable variable, void *value)
 {
-    printf("NPP_GETVALUE %d %x\n", (int) variable, (int)value);
+    Debug("NPP_GETVALUE %d %x\n", (int) variable, (int)value);
     switch(variable) {
     case NPPVpluginScriptableNPObject: {
 	*((NPObject **) value) = NPN.createobject(instance, &genresClass);
@@ -454,7 +461,7 @@ NPError NPP_SetWindow(NPP instance, NPWindow* npwindow)
 {
     Window xid= (Window)npwindow->window;
     
-    printf("NPP_SETWINDOW %x %x %x %x\n", (int) instance, (int)xid,
+    Debug("NPP_SETWINDOW %x %x %x %x\n", (int) instance, (int)xid,
            (int)npwindow->ws_info, (int)npwindow->type );
     
     if(Pd->xid!=xid) {
@@ -468,7 +475,7 @@ NPError NPP_SetWindow(NPP instance, NPWindow* npwindow)
 
 NPError NPP_NewStream(NPP instance, NPMIMEType type, NPStream* stream, NPBool seekable, uint16* stype)
 {
-    printf("NPP_NEWSTREAM %x %x\n", (int) instance, (int)seekable);
+    Debug("NPP_NEWSTREAM %x %x\n", (int) instance, (int)seekable);
     return NPERR_GENERIC_ERROR;
 }
 
@@ -493,7 +500,9 @@ PLEvent * strevent( PD *pd, const char * str)  // make event from a string (is c
     if(!pd->instance)	{ PR_Unlock(pd->lock); return 0; }
     ObserveEvent *event = PR_MALLOC(sizeof(ObserveEvent));
     if (!event)		{ PR_Unlock(pd->lock); return 0; }
-    printf("newEvent %u %s\n", (int)event, str);
+
+    if(strncmp(str,"debug=",6)==0) debug=str[6]-'0';
+    Debug("newEvent %u %s\n", (int)event, str);
     
     PL_InitEvent( &event->e, pd, HandleEvent, DestroyEvent);
     event->str=PR_Malloc(strlen(str)+3);
@@ -509,8 +518,8 @@ void PR_CALLBACK DestroyEvent(ObserveEvent* event)
 {
     if(event) {
 	PR_Lock(event->pd->lock);
-	printf("DestroyEvent %u ", (int)event);
-	printf("pdata refcnt=%d\n", event->pd->refcnt);
+	Debug("DestroyEvent %u ", (int)event);
+	Debug("pdata refcnt=%d\n", event->pd->refcnt);
 	free_pd(event->pd);
 	if(event->str) PR_Free(event->str);
 	PR_Free(event);
@@ -519,10 +528,10 @@ void PR_CALLBACK DestroyEvent(ObserveEvent* event)
 
 void * PR_CALLBACK HandleEvent(ObserveEvent* event)
 {
-    printf("HandleEvent %d \"%s\"\t", (int)event, event->str+2);
+    Debug("HandleEvent %d \"%s\"\t", (int)event, event->str+2);
     size_t namelen;
     for( namelen=2 ; ; namelen++ ) {
-    if( !event->str[namelen] ) { printf("I\n");		return 0; }
+    if( !event->str[namelen] ) { Debug("I\n");		return 0; }
     else if( event->str[namelen] == ':'  ||  event->str[namelen] == '=' ) break;
 	else if( event->str[namelen] & 128 ) event->str[namelen]=' ';
     }
@@ -530,13 +539,14 @@ void * PR_CALLBACK HandleEvent(ObserveEvent* event)
     char *name=strrchr(event->str+2,' ');
     if(!name++) name=event->str+2;
     if(!*name) {
-	printf("I\n");		return 0;
+	Debug("I\n");		return 0;
     }
     NPVariant val;
     STRINGZ_TO_NPVARIANT( event->str+namelen+1, val );
     NPObject *obj; 
     if(event->pd->instance && NPN.getvalue(event->pd->instance, NPNVPluginElementNPObject, &obj)==NPERR_NO_ERROR) {
 	if( storage_set( &(event->pd->vars), NPN.getstringidentifier(name), &val ) ) {
+	    if(strcmp(name,"debug")==0) debug=event->str[namelen+1]-'0';
 	    // invoke a callback function
 	    name-=2; name[0]='o'; name[1]='n';
 	    int i;
@@ -547,18 +557,18 @@ void * PR_CALLBACK HandleEvent(ObserveEvent* event)
 	    
 	    if(!(   NPN.invoke(event->pd->instance, obj, id0, &val, 0, &val)
 	         || NPN.invoke(event->pd->instance, obj, id1, &val, 0, &val)
-		)&&( printf("getprop "), NPN.getproperty(event->pd->instance, obj, id0, &val) && NPVARIANT_IS_STRING(val)
+		)&&( Debug("getprop "), NPN.getproperty(event->pd->instance, obj, id0, &val) && NPVARIANT_IS_STRING(val)
 	         || (NPN.releasevariantvalue(&val), NPN.getproperty(event->pd->instance, obj, id1, &val) && NPVARIANT_IS_STRING(val))
 		)
 	    ) {
-		printf("evaluate ");
+		Debug("evaluate ");
 		//In "evaluate" the instance can be desroyed
 		//NPN.evaluate(event->pd->instance, obj, &(val.value.stringValue), 0);
 		char *url=PR_Malloc(val.value.stringValue.utf8length+12);
 		strcpy(url,"javascript:");
 		strncpy(url+11,val.value.stringValue.utf8characters, val.value.stringValue.utf8length);
 		url[val.value.stringValue.utf8length+11]=0;
-		printf("geturl '%s' ",url);
+		Debug("geturl '%s' ",url);
 		NPN.geturl(event->pd->instance, url, "_self");
 		PR_Free(url);
 	    }
@@ -567,7 +577,7 @@ void * PR_CALLBACK HandleEvent(ObserveEvent* event)
 	    NPN.releasevariantvalue(&val);
 	    
 	}
-	printf("S\n");
-    }else printf("E\n");
+	Debug("S\n");
+    }else Debug("E\n");
     return 0;
 }
