@@ -1,4 +1,4 @@
-// $Id: observer_thread.c,v 1.2 2006/02/03 21:43:08 khlut Exp $
+// $Id: observer_thread.c,v 1.3 2006/02/13 17:42:46 khlut Exp $
 // Copyright 2005 Elphel, Inc.
 //
 // This file is part of GenReS.
@@ -40,8 +40,8 @@ int run_slave( PD * pd, char * mime, int restart ) // this function is not in ob
 	if(restart) execl(path,path,"--no-vars",0);
 	else execl(path,path,0);
 	perror("error");
-	printf("errorfile=%s\n",path);
-	printf("run=0\n");	
+	Debug("errorfile=%s\n",path);
+	Debug("run=0\n");	
 	_exit(1);
     }
     if(pd->slave==-1) return 1;
@@ -65,7 +65,7 @@ void PR_CALLBACK observer_thread( PD * pd )
 	i=strlen(buf);
         if(buf[--i]=='\n')  buf[i]=0;
 	if(!i)		continue;
-	while(pd->refcnt>30 && pd->status!=Cancelling) {PR_Sleep(100); printf("wait main thread\n"); continue;}
+	while(pd->refcnt>30 && pd->status!=Cancelling) {PR_Sleep(100); Debug("wait main thread\n"); continue;}
 	while(pd->status!=Cancelling && !post_event_to_thread( pd->browser, strevent, pd, buf )) PR_Sleep(100);
     } 
     PR_Lock(pd->lock);
