@@ -1,4 +1,4 @@
-// $Id: threadqueue.cpp,v 1.1.1.1 2005/11/04 09:07:01 khlut Exp $
+// $Id: threadqueue.cpp,v 1.3 2006/02/13 17:42:46 khlut Exp $
 // Copyright 2005 Elphel, Inc.
 //
 // This file is part of GenReS.
@@ -28,17 +28,46 @@ static NS_DEFINE_CID( kEventQueueService, NS_EVENTQUEUESERVICE_CID );
 extern "C" void PR_CALLBACK DestroyEvent(PLEvent* event) ;                                                                                   
 extern "C" void * PR_CALLBACK HandleEvent(PLEvent* event) ;
 
+nsIEventQueueService * eventQService=0;
+
+extern "C" void init_qservice(nsISupports *nss)
+{
+    nsIServiceManager * SM = 0;
+
+    if(!nss)	return;
+
+    nss->QueryInterface(NS_GET_IID(nsIServiceManager), (void**)&SM);
+    NS_RELEASE(nss);
+
+    if(!SM) {
+        printf("Unable to get service manager!\n");	return;
+    }
+
+    SM->GetService(kEventQueueService, NS_GET_IID(nsIEventQueueService), (void **)&eventQService);
+    NS_RELEASE(SM);
+
+    if(!eventQService) {
+        printf("Unable to get event queue service!\n");	return;
+    }
+}
+extern "C" void shut_qservice()
+{
+    NS_IF_RELEASE(eventQService);
+    return;
+}
 
 extern "C" bool post_event_to_thread( PRThread *receiver, PLEvent *(*allocator)(void * owner, const void * msg), 
 void * owner, const void * msg )
 {
-    nsresult rv; bool r=true;
+    bool r=true;
     PLEvent * event;
 
-    nsCOMPtr<nsIEventQueueService> eventQService = do_GetService(kEventQueueService, &rv);
-    if(NS_FAILED(rv))	return false;
+    if(!eventQService)		return false;
+
     nsCOMPtr<nsIEventQueue> eventQ;
-    if(NS_FAILED( eventQService->GetThreadEventQueue(receiver,getter_AddRefs(eventQ)) ))	return false;
+    if(NS_FAILED( eventQService->GetThreadEventQueue(receiver,getter_AddRefs(eventQ)) )) {
+	return false;
+    }
     eventQ->EnterMonitor();
     if((event=(*allocator)(owner, msg)))
       if(NS_FAILED( eventQ->PostEvent(event) )) {
@@ -47,7 +76,7 @@ void * owner, const void * msg )
       }
     eventQ->ExitMonitor();
 
-    return true;
+    return r;
 }
 
 extern "C" void ProcessOwnerEvents(PLEvent* event, void* data, PLEventQueue* queue) {
@@ -59,14 +88,9 @@ extern "C" void ProcessOwnerEvents(PLEvent* event, void* data, PLEventQueue* que
 
 extern "C" bool process_pending_events(void *owner)
 {
-    nsresult rv;
-
     // Get the event queue of the current thread...
-    nsCOMPtr<nsIEventQueueService> eventQService = do_GetService(kEventQueueService, &rv);
-    if(NS_FAILED(rv)) {
-        printf("Unable to get event queue service!\n");
-        return false;
-    }
+
+    if(!eventQService)		return false;
 
     nsCOMPtr<nsIEventQueue> eventQ;
      
